@@ -35,6 +35,20 @@
 #define DEFAULT_COMMENT_STRING           "#"
 #define DEFAULT_MAX_FILE_SIZE_FOR_MMAP (4LL*1024LL*1024LL*1024LL)
 
+// ASCII 1f and 1e
+#define ASV_FS "\x1f"
+#define ASV_RS "\x1e"
+
+#define ASV_FS_FOR_HELP "0x1f"
+#define ASV_RS_FOR_HELP "0x1e"
+
+// Unicode code points U+241F and U+241E, encoded as UTF-8.
+#define USV_FS "\xe2\x90\x9f"
+#define USV_RS "\xe2\x90\x9e"
+
+#define USV_FS_FOR_HELP "U+241F (UTF-8 0xe2909f)"
+#define USV_RS_FOR_HELP "U+241E (UTF-8 0xe2909e)"
+
 // ----------------------------------------------------------------
 static mapper_setup_t* mapper_lookup_table[] = {
 
@@ -50,6 +64,7 @@ static mapper_setup_t* mapper_lookup_table[] = {
 	&mapper_decimate_setup,
 	&mapper_fill_down_setup,
 	&mapper_filter_setup,
+	&mapper_format_values_setup,
 	&mapper_fraction_setup,
 	&mapper_grep_setup,
 	&mapper_group_by_setup,
@@ -66,6 +81,7 @@ static mapper_setup_t* mapper_lookup_table[] = {
 	&mapper_nothing_setup,
 	&mapper_put_setup,
 	&mapper_regularize_setup,
+	&mapper_remove_empty_columns_setup,
 	&mapper_rename_setup,
 	&mapper_reorder_setup,
 	&mapper_repeat_setup,
@@ -75,7 +91,10 @@ static mapper_setup_t* mapper_lookup_table[] = {
 	&mapper_sec2gmtdate_setup,
 	&mapper_seqgen_setup,
 	&mapper_shuffle_setup,
+	&mapper_skip_trivial_records_setup,
 	&mapper_sort_setup,
+	// xxx temp for 5.4.0 -- will continue work after
+	// &mapper_sort_within_records_setup,
 	&mapper_stats1_setup,
 	&mapper_stats2_setup,
 	&mapper_step_setup,
@@ -250,15 +269,16 @@ cli_opts_t* parse_command_line(int argc, char** argv, sllv_t** ppmapper_list) {
 	// mappers operate on all input files. Also retain information needed to construct them
 	// for each input file, for in-place mode.
 	popts->mapper_argb = argi;
-	popts->argv = argv;
+	popts->original_argv = argv;
+	popts->non_in_place_argv = copy_argv(argv);
 	popts->argc = argc;
-	*ppmapper_list = cli_parse_mappers(argv, &argi, argc, popts, &no_input);
+	*ppmapper_list = cli_parse_mappers(popts->non_in_place_argv, &argi, argc, popts, &no_input);
 
 	for ( ; argi < argc; argi++) {
 		slls_append(popts->filenames, argv[argi], NO_FREE);
 	}
 
-	// Check for use of mmap. It's fractionally faster than stdio (due to fewer data copies
+	// Check for use of mmap. It's about 20% faster than stdio (due to fewer data copies
 	// -- lrecs can be pointer-backed by mmap memory) but we can't use it in all situations.
 	if (no_input) {
 		slls_free(popts->filenames);
@@ -266,6 +286,10 @@ cli_opts_t* parse_command_line(int argc, char** argv, sllv_t** ppmapper_list) {
 	} else if (popts->filenames->length == 0) {
 		// No filenames means read from standard input, and standard input cannot be mmapped.
 		popts->reader_opts.use_mmap_for_read = FALSE;
+	} else if (popts->filenames->length > 10) {
+		// https://github.com/johnkerl/miller/issues/256: too many small files is as bad as one big one
+		// (for which see immediately below).
+		popts->reader_opts.use_mmap_for_read = FALSE;
 	} else if (popts->reader_opts.use_mmap_for_read == TRUE) {
 		// https://github.com/johnkerl/miller/issues/160: don't use mmap for large files.
 		//
@@ -318,6 +342,14 @@ sllv_t* cli_parse_mappers(char** argv, int* pargi, int argc, cli_opts_t* popts,
 		main_usage_short(stderr, MLR_GLOBALS.bargv0);
 		exit(1);
 	}
+
+	// Note that the command-line parsers can operate destructively on argv, e.g. verbs
+	// which take comma-delimited field names splitting on commas.  For this reason we
+	// need to duplicate argv on each in-place run within the streamer module. But before
+	// that ever happens, here we run through the verb-parsers once to find out where it
+	// is on the command line that the verbs and their arguments end and the filenames
+	// begin.
+
 	while (TRUE) {
 		check_arg_count(argv, argi, argc, 1);
 		char* verb = argv[argi];
@@ -367,6 +399,7 @@ void cli_opts_free(cli_opts_t* popts) {
 		return;
 
 	slls_free(popts->filenames);
+	free_argv_copy(popts->non_in_place_argv);
 	free(popts);
 	free_opt_singletons();
 }
@@ -754,6 +787,12 @@ static void main_usage_data_format_options(FILE* o, char* argv0) {
 	fprintf(o, "\n");
 	fprintf(o, "  --itsv    --otsv    --tsv       Keystroke-savers for \"--icsv --ifs tab\",\n");
 	fprintf(o, "                                  \"--ocsv --ofs tab\", \"--csv --fs tab\".\n");
+	fprintf(o, "  --iasv    --oasv    --asv       Similar but using ASCII FS %s and RS %s\n",
+		ASV_FS_FOR_HELP, ASV_RS_FOR_HELP);
+	fprintf(o, "  --iusv    --ousv    --usv       Similar but using Unicode FS %s\n",
+		USV_FS_FOR_HELP);
+	fprintf(o, "                                  and RS %s\n",
+		USV_RS_FOR_HELP);
 	fprintf(o, "\n");
 	fprintf(o, "  --icsvlite --ocsvlite --csvlite Comma-separated value (or tab-separated\n");
 	fprintf(o, "                                  with --fs tab, etc.). The 'lite' CSV does not handle\n");
@@ -765,6 +804,12 @@ static void main_usage_data_format_options(FILE* o, char* argv0) {
 	fprintf(o, "  --itsvlite --otsvlite --tsvlite Keystroke-savers for \"--icsvlite --ifs tab\",\n");
 	fprintf(o, "                                  \"--ocsvlite --ofs tab\", \"--csvlite --fs tab\".\n");
 	fprintf(o, "  -t                              Synonymous with --tsvlite.\n");
+	fprintf(o, "  --iasvlite --oasvlite --asvlite Similar to --itsvlite et al. but using ASCII FS %s and RS %s\n",
+		ASV_FS_FOR_HELP, ASV_RS_FOR_HELP);
+	fprintf(o, "  --iusvlite --ousvlite --usvlite Similar to --itsvlite et al. but using Unicode FS %s\n",
+		USV_FS_FOR_HELP);
+	fprintf(o, "                                  and RS %s\n",
+		USV_RS_FOR_HELP);
 	fprintf(o, "\n");
 	fprintf(o, "  --ipprint --opprint --pprint    Pretty-printed tabular (produces no\n");
 	fprintf(o, "                                  output until all input is in).\n");
@@ -807,6 +852,10 @@ static void main_usage_data_format_options(FILE* o, char* argv0) {
 	fprintf(o, "\n");
 	fprintf(o, "  Examples: --csv for CSV-formatted input and output; --idkvp --opprint for\n");
 	fprintf(o, "  DKVP-formatted input and pretty-printed output.\n");
+	fprintf(o, "\n");
+	fprintf(o, "  Please use --iformat1 --oformat2 rather than --format1 --oformat2.\n");
+	fprintf(o, "  The latter sets up input and output flags for format1, not all of which\n");
+	fprintf(o, "  are overridden in all cases by setting output format to format2.\n");
 }
 
 static void main_usage_comments_in_data(FILE* o, char* argv0) {
@@ -832,7 +881,7 @@ static void main_usage_comments_in_data(FILE* o, char* argv0) {
 
 static void main_usage_format_conversion_keystroke_saver_options(FILE* o, char* argv0) {
 	fprintf(o, "As keystroke-savers for format-conversion you may use the following:\n");
-	fprintf(o, "  --c2t --c2d --c2n --c2j --c2x --c2p --c2m\n");
+	fprintf(o, "        --c2t --c2d --c2n --c2j --c2x --c2p --c2m\n");
 	fprintf(o, "  --t2c       --t2d --t2n --t2j --t2x --t2p --t2m\n");
 	fprintf(o, "  --d2c --d2t       --d2n --d2j --d2x --d2p --d2m\n");
 	fprintf(o, "  --n2c --n2t --n2d       --n2j --n2x --n2p --n2m\n");
@@ -930,7 +979,12 @@ static void main_usage_csv_options(FILE* o, char* argv0) {
 	fprintf(o, "  --implicit-csv-header Use 1,2,3,... as field labels, rather than from line 1\n");
 	fprintf(o, "                     of input files. Tip: combine with \"label\" to recreate\n");
 	fprintf(o, "                     missing headers.\n");
+	fprintf(o, "  --allow-ragged-csv-input|--ragged If a data line has fewer fields than the header line,\n");
+	fprintf(o, "                     fill remaining keys with empty string. If a data line has more\n");
+	fprintf(o, "                     fields than the header line, use integer field labels as in\n");
+	fprintf(o, "                     the implicit-header case.\n");
 	fprintf(o, "  --headerless-csv-output   Print only CSV data lines.\n");
+	fprintf(o, "  -N                 Keystroke-saver for --implicit-csv-header --headerless-csv-output.\n");
 }
 
 static void main_usage_double_quoting(FILE* o, char* argv0) {
@@ -1084,6 +1138,7 @@ void cli_reader_opts_init(cli_reader_opts_t* preader_opts) {
 	preader_opts->allow_repeat_ifs               = NEITHER_TRUE_NOR_FALSE;
 	preader_opts->allow_repeat_ips               = NEITHER_TRUE_NOR_FALSE;
 	preader_opts->use_implicit_csv_header        = NEITHER_TRUE_NOR_FALSE;
+	preader_opts->allow_ragged_csv_input         = NEITHER_TRUE_NOR_FALSE;
 	preader_opts->use_mmap_for_read              = NEITHER_TRUE_NOR_FALSE;
 
 	preader_opts->prepipe                        = NULL;
@@ -1140,6 +1195,9 @@ void cli_apply_reader_defaults(cli_reader_opts_t* preader_opts) {
 	if (preader_opts->use_implicit_csv_header == NEITHER_TRUE_NOR_FALSE)
 		preader_opts->use_implicit_csv_header = FALSE;
 
+	if (preader_opts->allow_ragged_csv_input == NEITHER_TRUE_NOR_FALSE)
+		preader_opts->allow_ragged_csv_input = FALSE;
+
 	if (preader_opts->use_mmap_for_read == NEITHER_TRUE_NOR_FALSE)
 #if MLR_ARCH_MMAP_ENABLED
 		preader_opts->use_mmap_for_read = TRUE;
@@ -1250,6 +1308,9 @@ void cli_merge_reader_opts(cli_reader_opts_t* pfunc_opts, cli_reader_opts_t* pma
 	if (pfunc_opts->use_implicit_csv_header == NEITHER_TRUE_NOR_FALSE)
 		pfunc_opts->use_implicit_csv_header = pmain_opts->use_implicit_csv_header;
 
+	if (pfunc_opts->allow_ragged_csv_input == NEITHER_TRUE_NOR_FALSE)
+		pfunc_opts->allow_ragged_csv_input = pmain_opts->allow_ragged_csv_input;
+
 	if (pfunc_opts->use_mmap_for_read == NEITHER_TRUE_NOR_FALSE)
 		pfunc_opts->use_mmap_for_read = pmain_opts->use_mmap_for_read;
 
@@ -1460,6 +1521,10 @@ int cli_handle_reader_options(char** argv, int argc, int *pargi, cli_reader_opts
 		preader_opts->use_implicit_csv_header = TRUE;
 		argi += 1;
 
+	} else if (streq(argv[argi], "--allow-ragged-csv-input") || streq(argv[argi], "--ragged")) {
+		preader_opts->allow_ragged_csv_input = TRUE;
+		argi += 1;
+
 	} else if (streq(argv[argi], "--ips")) {
 		check_arg_count(argv, argi, argc, 2);
 		preader_opts->ips = cli_sep_from_arg(argv[argi+1]);
@@ -1521,6 +1586,30 @@ int cli_handle_reader_options(char** argv, int argc, int *pargi, cli_reader_opts
 		preader_opts->ifs = "\t";
 		argi += 1;
 
+	} else if (streq(argv[argi], "--iasv")) {
+		preader_opts->ifile_fmt = "csv";
+		preader_opts->ifs = ASV_FS;
+		preader_opts->irs = ASV_RS;
+		argi += 1;
+
+	} else if (streq(argv[argi], "--iasvlite")) {
+		preader_opts->ifile_fmt = "csvlite";
+		preader_opts->ifs = ASV_FS;
+		preader_opts->irs = ASV_RS;
+		argi += 1;
+
+	} else if (streq(argv[argi], "--iusv")) {
+		preader_opts->ifile_fmt = "csv";
+		preader_opts->ifs = USV_FS;
+		preader_opts->irs = USV_RS;
+		argi += 1;
+
+	} else if (streq(argv[argi], "--iusvlite")) {
+		preader_opts->ifile_fmt = "csvlite";
+		preader_opts->ifs = USV_FS;
+		preader_opts->irs = USV_RS;
+		argi += 1;
+
 	} else if (streq(argv[argi], "--idkvp")) {
 		preader_opts->ifile_fmt = "dkvp";
 		argi += 1;
@@ -1674,6 +1763,30 @@ int cli_handle_writer_options(char** argv, int argc, int *pargi, cli_writer_opts
 		pwriter_opts->ofs = "\t";
 		argi += 1;
 
+	} else if (streq(argv[argi], "--oasv")) {
+		pwriter_opts->ofile_fmt = "csv";
+		pwriter_opts->ofs = ASV_FS;
+		pwriter_opts->ors = ASV_RS;
+		argi += 1;
+
+	} else if (streq(argv[argi], "--oasvlite")) {
+		pwriter_opts->ofile_fmt = "csvlite";
+		pwriter_opts->ofs = ASV_FS;
+		pwriter_opts->ors = ASV_RS;
+		argi += 1;
+
+	} else if (streq(argv[argi], "--ousv")) {
+		pwriter_opts->ofile_fmt = "csv";
+		pwriter_opts->ofs = USV_FS;
+		pwriter_opts->ors = USV_RS;
+		argi += 1;
+
+	} else if (streq(argv[argi], "--ousvlite")) {
+		pwriter_opts->ofile_fmt = "csvlite";
+		pwriter_opts->ofs = USV_FS;
+		pwriter_opts->ors = USV_RS;
+		argi += 1;
+
 	} else if (streq(argv[argi], "--omd")) {
 		pwriter_opts->ofile_fmt = "markdown";
 		argi += 1;
@@ -1803,6 +1916,38 @@ int cli_handle_reader_writer_options(char** argv, int argc, int *pargi,
 		pwriter_opts->ofs = "\t";
 		argi += 1;
 
+	} else if (streq(argv[argi], "--asv")) {
+		preader_opts->ifile_fmt = pwriter_opts->ofile_fmt = "csv";
+		preader_opts->ifs = ASV_FS;
+		pwriter_opts->ofs = ASV_FS;
+		preader_opts->irs = ASV_RS;
+		pwriter_opts->ors = ASV_RS;
+		argi += 1;
+
+	} else if (streq(argv[argi], "--asvlite")) {
+		preader_opts->ifile_fmt = pwriter_opts->ofile_fmt = "csvlite";
+		preader_opts->ifs = ASV_FS;
+		pwriter_opts->ofs = ASV_FS;
+		preader_opts->irs = ASV_RS;
+		pwriter_opts->ors = ASV_RS;
+		argi += 1;
+
+	} else if (streq(argv[argi], "--usv")) {
+		preader_opts->ifile_fmt = pwriter_opts->ofile_fmt = "csv";
+		preader_opts->ifs = USV_FS;
+		pwriter_opts->ofs = USV_FS;
+		preader_opts->irs = USV_RS;
+		pwriter_opts->ors = USV_RS;
+		argi += 1;
+
+	} else if (streq(argv[argi], "--usvlite")) {
+		preader_opts->ifile_fmt = pwriter_opts->ofile_fmt = "csvlite";
+		preader_opts->ifs = USV_FS;
+		pwriter_opts->ofs = USV_FS;
+		preader_opts->irs = USV_RS;
+		pwriter_opts->ors = USV_RS;
+		argi += 1;
+
 	} else if (streq(argv[argi], "--dkvp")) {
 		preader_opts->ifile_fmt = "dkvp";
 		pwriter_opts->ofile_fmt = "dkvp";
@@ -2093,6 +2238,10 @@ int cli_handle_reader_writer_options(char** argv, int argc, int *pargi,
 		pwriter_opts->ofile_fmt = "markdown";
 		argi += 1;
 
+	} else if (streq(argv[argi], "-N")) {
+		preader_opts->use_implicit_csv_header = TRUE;
+		pwriter_opts->headerless_csv_output = TRUE;
+		argi += 1;
 	}
 	*pargi = argi;
 	return argi != oargi;
