@@ -125,9 +125,8 @@ static int lhms2v_find_index_for_key(lhms2v_t* pmap, char* key1, char* key2, int
 		if (++index >= pmap->array_length)
 			index = 0;
 	}
-	fprintf(stderr, "%s: internal coding error detected in file %s at line %d.\n",
-		MLR_GLOBALS.bargv0, __FILE__, __LINE__);
-	exit(1);
+	MLR_INTERNAL_CODING_ERROR();
+	return -1; // not reached
 }
 
 // ----------------------------------------------------------------
