@@ -241,11 +241,13 @@ static function_lookup_t FUNCTION_LOOKUP_TABLE[] = {
 		"inclusive. Negative indices -len .. -1 alias to 0 .. len-1."},
 	{FUNC_CLASS_STRING, "tolower",  1,0, "Convert string to lowercase."},
 	{FUNC_CLASS_STRING, "toupper",  1,0, "Convert string to uppercase."},
+	{FUNC_CLASS_STRING, "capitalize",  1,0, "Convert string's first character to uppercase."},
 	{FUNC_CLASS_STRING, "lstrip",  1,0,  "Strip leading whitespace from string."},
 	{FUNC_CLASS_STRING, "rstrip",  1,0,  "Strip trailing whitespace from string."},
 	{FUNC_CLASS_STRING, "strip",  1,0,  "Strip leading and trailing whitespace from string."},
 	{FUNC_CLASS_STRING, "collapse_whitespace",  1,0,  "Strip repeated whitespace from string."},
 	{FUNC_CLASS_STRING, "clean_whitespace",  1,0,  "Same as collapse_whitespace and strip."},
+	{FUNC_CLASS_STRING, "system",  1,0, "Run command string, yielding its stdout minus final carriage return."},
 
 	{FUNC_CLASS_MATH, "abs",      1,0, "Absolute value."},
 	{FUNC_CLASS_MATH, "acos",     1,0, "Inverse trigonometric cosine."},
@@ -289,8 +291,10 @@ static function_lookup_t FUNCTION_LOOKUP_TABLE[] = {
 	{FUNC_CLASS_MATH, "tan",      1,0, "Trigonometric tangent."},
 	{FUNC_CLASS_MATH, "tanh",     1,0, "Hyperbolic tangent."},
 	{FUNC_CLASS_MATH, "urand",    0,0,
-		"Floating-point numbers on the unit interval.\n"
+		"Floating-point numbers uniformly distributed on the unit interval.\n"
 		"Int-valued example: '$n=floor(20+urand()*11)'." },
+	{FUNC_CLASS_MATH, "urandrange",    2,0,
+		"Floating-point numbers uniformly distributed on the interval [a, b)." },
 	{FUNC_CLASS_MATH, "urand32",  0,0, "Integer uniformly distributed 0 and 2**32-1\n"
 	"inclusive." },
 	{FUNC_CLASS_MATH, "urandint", 2,0, "Integer uniformly distributed between inclusive\ninteger endpoints." },
@@ -371,7 +375,7 @@ static function_lookup_t FUNCTION_LOOKUP_TABLE[] = {
 	{FUNC_CLASS_TYPING, "is_int",         1,0, "True if field is present with value inferred to be int "},
 	{FUNC_CLASS_TYPING, "is_map",         1,0, "True if argument is a map."},
 	{FUNC_CLASS_TYPING, "is_nonempty_map", 1,0, "True if argument is a map which is non-empty."},
-	{FUNC_CLASS_TYPING, "is_not_empty",    1,0, "False if field is present in input with empty value, false otherwise"},
+	{FUNC_CLASS_TYPING, "is_not_empty",    1,0, "False if field is present in input with empty value, true otherwise"},
 	{FUNC_CLASS_TYPING, "is_not_map",      1,0, "True if argument is not a map."},
 	{FUNC_CLASS_TYPING, "is_not_null",     1,0, "False if argument is null (empty or absent), true otherwise."},
 	{FUNC_CLASS_TYPING, "is_null",        1,0, "True if argument is null (empty or absent), false otherwise."},
@@ -1202,6 +1206,8 @@ static rval_evaluator_t* fmgr_alloc_evaluator_from_unary_func_name(char* fnnm, r
 	} else if (streq(fnnm, "tanh"))            { return rval_evaluator_alloc_from_f_f_func(f_f_tanh_func,        parg1);
 	} else if (streq(fnnm, "tolower"))         { return rval_evaluator_alloc_from_s_s_func(s_s_tolower_func,     parg1);
 	} else if (streq(fnnm, "toupper"))         { return rval_evaluator_alloc_from_s_s_func(s_s_toupper_func,     parg1);
+	} else if (streq(fnnm, "capitalize"))      { return rval_evaluator_alloc_from_s_s_func(s_s_capitalize_func,  parg1);
+	} else if (streq(fnnm, "system"))          { return rval_evaluator_alloc_from_s_s_func(s_s_system_func,      parg1);
 	} else if (streq(fnnm, "lstrip"))          { return rval_evaluator_alloc_from_s_s_func(s_s_lstrip_func,      parg1);
 	} else if (streq(fnnm, "rstrip"))          { return rval_evaluator_alloc_from_s_s_func(s_s_rstrip_func,      parg1);
 	} else if (streq(fnnm, "strip"))           { return rval_evaluator_alloc_from_s_s_func(s_s_strip_func,       parg1);
@@ -1262,6 +1268,7 @@ static rval_evaluator_t* fmgr_alloc_evaluator_from_binary_func_name(char* fnnm,
 	} else if (streq(fnnm, "strftime_local")) { return rval_evaluator_alloc_from_x_ns_func(s_ns_strftime_local_func, parg1, parg2);
 	} else if (streq(fnnm, "strptime")) { return rval_evaluator_alloc_from_x_ss_func(i_ss_strptime_func, parg1, parg2);
 	} else if (streq(fnnm, "strptime_local")) { return rval_evaluator_alloc_from_x_ss_func(i_ss_strptime_local_func, parg1, parg2);
+	} else if (streq(fnnm, "urandrange")) { return rval_evaluator_alloc_from_f_ff_func(f_ff_urandrange_func, parg1, parg2);
 	} else  { return NULL; }
 }
 
