@@ -34,14 +34,13 @@
 // ----------------------------------------------------------------
 #define STRING_BUILDER_INIT_SIZE 1024
 
-// AKA "token"
-#define IRS_STRIDX           0x2001
-#define IFS_STRIDX           0x2002
-#define DQUOTE_STRIDX        0x2003
-#define DQUOTE_IRS_STRIDX    0x2004
-#define DQUOTE_IRS2_STRIDX   0x2005 // alternate line-ending for autodetect LF/CRLF
-#define DQUOTE_IFS_STRIDX    0x2006
-#define DQUOTE_DQUOTE_STRIDX 0x2007
+#define IRS_TOKEN           0x2001
+#define IFS_TOKEN           0x2002
+#define DQUOTE_TOKEN        0x2003
+#define DQUOTE_IRS_TOKEN    0x2004
+#define DQUOTE_IRS2_TOKEN   0x2005 // alternate line-ending for autodetect LF/CRLF
+#define DQUOTE_IFS_TOKEN    0x2006
+#define DQUOTE_DQUOTE_TOKEN 0x2007
 
 // ----------------------------------------------------------------
 typedef struct _lrec_reader_mmap_csv_state_t {
@@ -73,7 +72,8 @@ typedef struct _lrec_reader_mmap_csv_state_t {
 	parse_trie_t*       pdquote_parse_trie;
 
 	int                 expect_header_line_next;
-	int                 use_implicit_header;
+	int                 use_implicit_csv_header;
+	int                 allow_ragged_csv_input;
 	header_keeper_t*    pheader_keeper;
 	lhmslv_t*           pheader_keepers;
 
@@ -85,11 +85,14 @@ static lrec_t* lrec_reader_mmap_csv_process(void* pvstate, void* pvhandle, conte
 static int     lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
 	rslls_t* pfields, file_reader_mmap_state_t* phandle, context_t* pctx);
 static lrec_t* paste_indices_and_data(lrec_reader_mmap_csv_state_t* pstate, rslls_t* pdata_fields, context_t* pctx);
-static lrec_t* paste_header_and_data(lrec_reader_mmap_csv_state_t* pstate, rslls_t* pdata_fields, context_t* pctx);
+static lrec_t* paste_header_and_data_ragged(lrec_reader_mmap_csv_state_t* pstate, rslls_t* pdata_fields,
+	context_t* pctx);
+static lrec_t* paste_header_and_data_rectangular(lrec_reader_mmap_csv_state_t* pstate, rslls_t* pdata_fields,
+	context_t* pctx);
 
 // ----------------------------------------------------------------
-lrec_reader_t* lrec_reader_mmap_csv_alloc(char* irs, char* ifs, int use_implicit_header,
-	comment_handling_t comment_handling, char* comment_string)
+lrec_reader_t* lrec_reader_mmap_csv_alloc(char* irs, char* ifs, int use_implicit_csv_header,
+	int allow_ragged_csv_input, comment_handling_t comment_handling, char* comment_string)
 {
 	lrec_reader_t* plrec_reader = mlr_malloc_or_die(sizeof(lrec_reader_t));
 
@@ -119,29 +122,30 @@ lrec_reader_t* lrec_reader_mmap_csv_alloc(char* irs, char* ifs, int use_implicit
 	pstate->dquotelen     = strlen(pstate->dquote);
 
 	pstate->pno_dquote_parse_trie = parse_trie_alloc();
-	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->irs,     IRS_STRIDX);
-	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->ifs,     IFS_STRIDX);
-	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->dquote,  DQUOTE_STRIDX);
+	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->irs,     IRS_TOKEN);
+	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->ifs,     IFS_TOKEN);
+	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->dquote,  DQUOTE_TOKEN);
 
 	pstate->pdquote_parse_trie = parse_trie_alloc();
 	if (pstate->do_auto_line_term) {
 		pstate->dquote_irs  = mlr_paste_2_strings("\"", "\n");
 		pstate->dquote_irs2 = mlr_paste_2_strings("\"", "\r\n");
-		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs,  DQUOTE_IRS_STRIDX);
-		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs2, DQUOTE_IRS2_STRIDX);
+		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs,  DQUOTE_IRS_TOKEN);
+		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs2, DQUOTE_IRS2_TOKEN);
 	} else {
 		pstate->dquote_irs  = mlr_paste_2_strings("\"", pstate->irs);
 		pstate->dquote_irs2 = NULL;
-		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs, DQUOTE_IRS_STRIDX);
+		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs, DQUOTE_IRS_TOKEN);
 	}
-	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_ifs,    DQUOTE_IFS_STRIDX);
-	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_dquote, DQUOTE_DQUOTE_STRIDX);
+	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_ifs,    DQUOTE_IFS_TOKEN);
+	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_dquote, DQUOTE_DQUOTE_TOKEN);
 
 	pstate->pfields = rslls_alloc();
 	pstate->psb = sb_alloc(STRING_BUILDER_INIT_SIZE);
 
-	pstate->expect_header_line_next   = use_implicit_header ? FALSE : TRUE;
-	pstate->use_implicit_header       = use_implicit_header;
+	pstate->expect_header_line_next   = use_implicit_csv_header ? FALSE : TRUE;
+	pstate->use_implicit_csv_header   = use_implicit_csv_header;
+	pstate->allow_ragged_csv_input    = allow_ragged_csv_input;
 	pstate->pheader_keeper            = NULL;
 	pstate->pheader_keepers           = lhmslv_alloc();
 
@@ -179,7 +183,7 @@ static void lrec_reader_mmap_csv_free(lrec_reader_t* preader) {
 static void lrec_reader_mmap_csv_sof(void* pvstate, void* pvhandle) {
 	lrec_reader_mmap_csv_state_t* pstate = pvstate;
 	pstate->ilno = 0LL;
-	pstate->expect_header_line_next = pstate->use_implicit_header ? FALSE : TRUE;
+	pstate->expect_header_line_next = pstate->use_implicit_csv_header ? FALSE : TRUE;
 
 	// Strip UTF-8 BOM if any
 	file_reader_mmap_state_t* phandle = pvhandle;
@@ -290,9 +294,11 @@ static lrec_t* lrec_reader_mmap_csv_process(void* pvstate, void* pvhandle, conte
 			}
 		}
 
-		lrec_t* prec = pstate->use_implicit_header
+		lrec_t* prec = pstate->use_implicit_csv_header
 			? paste_indices_and_data(pstate, pstate->pfields, pctx)
-			: paste_header_and_data(pstate, pstate->pfields, pctx);
+			: pstate->allow_ragged_csv_input
+			? paste_header_and_data_ragged(pstate, pstate->pfields, pctx)
+			: paste_header_and_data_rectangular(pstate, pstate->pfields, pctx);
 		rslls_reset(pstate->pfields);
 		return prec;
 	}
@@ -301,7 +307,7 @@ static lrec_t* lrec_reader_mmap_csv_process(void* pvstate, void* pvhandle, conte
 static int lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
 	rslls_t* pfields, file_reader_mmap_state_t* phandle, context_t* pctx)
 {
-	int rc, stridx, matchlen, record_done, field_done;
+	int rc, token = 0, matchlen = 0, record_done = FALSE, field_done = FALSE;
 	string_builder_t* psb = pstate->psb;
 
 	if (phandle->sol >= phandle->eof)
@@ -320,16 +326,16 @@ static int lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
 			field_done = FALSE;
 			while (!field_done) {
 				MLR_INTERNAL_CODING_ERROR_IF(e > phandle->eof);
-				rc = parse_trie_match(pstate->pno_dquote_parse_trie, e, phandle->eof, &stridx, &matchlen);
+				rc = parse_trie_match(pstate->pno_dquote_parse_trie, e, phandle->eof, &token, &matchlen);
 				if (rc) {
-					switch(stridx) {
-					case IFS_STRIDX: // end of field
+					switch(token) {
+					case IFS_TOKEN: // end of field
 						*e = 0;
 						rslls_append(pfields, p, NO_FREE, 0);
 						p = e + matchlen;
 						field_done  = TRUE;
 						break;
-					case IRS_STRIDX: // end of record
+					case IRS_TOKEN: // end of record
 						*e = 0;
 
 						if (pstate->do_auto_line_term) {
@@ -346,14 +352,14 @@ static int lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
 						field_done  = TRUE;
 						record_done = TRUE;
 						break;
-					case DQUOTE_STRIDX: // CSV syntax error: fields containing quotes must be fully wrapped in quotes
+					case DQUOTE_TOKEN: // CSV syntax error: fields containing quotes must be fully wrapped in quotes
 						fprintf(stderr, "%s: syntax error: unwrapped double quote at line %lld.\n",
 							MLR_GLOBALS.bargv0, pstate->ilno);
 						exit(1);
 						break;
 					default:
 						fprintf(stderr, "%s: internal coding error: unexpected token %d at line %lld.\n",
-							MLR_GLOBALS.bargv0, stridx, pstate->ilno);
+							MLR_GLOBALS.bargv0, token, pstate->ilno);
 						exit(1);
 						break;
 					}
@@ -394,11 +400,11 @@ static int lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
 					exit(1);
 				}
 
-				rc = parse_trie_match(pstate->pdquote_parse_trie, e, phandle->eof, &stridx, &matchlen);
+				rc = parse_trie_match(pstate->pdquote_parse_trie, e, phandle->eof, &token, &matchlen);
 
 				if (rc) {
-					switch(stridx) {
-					case DQUOTE_IFS_STRIDX: // end of field
+					switch(token) {
+					case DQUOTE_IFS_TOKEN: // end of field
 						*e = 0;
 						if (contiguous)
 							rslls_append(pfields, p, NO_FREE, FIELD_QUOTED_ON_INPUT);
@@ -407,8 +413,8 @@ static int lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
 						p = e + matchlen;
 						field_done  = TRUE;
 						break;
-					case DQUOTE_IRS_STRIDX: // end of record
-					case DQUOTE_IRS2_STRIDX: // end of record
+					case DQUOTE_IRS_TOKEN: // end of record
+					case DQUOTE_IRS2_TOKEN: // end of record
 						*e = 0;
 
 						if (pstate->do_auto_line_term) {
@@ -428,7 +434,7 @@ static int lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
 						field_done  = TRUE;
 						record_done = TRUE;
 						break;
-					case DQUOTE_DQUOTE_STRIDX: // RFC-4180 CSV: "" inside a dquoted field is an escape for "
+					case DQUOTE_DQUOTE_TOKEN: // RFC-4180 CSV: "" inside a dquoted field is an escape for "
 						if (contiguous) { // not anymore it isn't
 							sb_append_char_range(psb, p, e);
 							contiguous = FALSE;
@@ -438,7 +444,7 @@ static int lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
 						break;
 					default:
 						fprintf(stderr, "%s: internal coding error: unexpected token %d at line %lld.\n",
-							MLR_GLOBALS.bargv0, stridx, pstate->ilno);
+							MLR_GLOBALS.bargv0, token, pstate->ilno);
 						exit(1);
 						break;
 					}
@@ -457,7 +463,9 @@ static int lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
 }
 
 // ----------------------------------------------------------------
-static lrec_t* paste_indices_and_data(lrec_reader_mmap_csv_state_t* pstate, rslls_t* pdata_fields, context_t* pctx) {
+static lrec_t* paste_indices_and_data(lrec_reader_mmap_csv_state_t* pstate, rslls_t* pdata_fields,
+	context_t* pctx)
+{
 	int idx = 0;
 	lrec_t* prec = lrec_unbacked_alloc();
 	for (rsllse_t* pd = pdata_fields->phead; idx < pdata_fields->length && pd != NULL; pd = pd->pnext) {
@@ -473,7 +481,53 @@ static lrec_t* paste_indices_and_data(lrec_reader_mmap_csv_state_t* pstate, rsll
 }
 
 // ----------------------------------------------------------------
-static lrec_t* paste_header_and_data(lrec_reader_mmap_csv_state_t* pstate, rslls_t* pdata_fields, context_t* pctx) {
+static lrec_t* paste_header_and_data_ragged(lrec_reader_mmap_csv_state_t* pstate, rslls_t* pdata_fields,
+	context_t* pctx)
+{
+	lrec_t* prec = lrec_unbacked_alloc();
+	sllse_t* ph  = pstate->pheader_keeper->pkeys->phead;
+	rsllse_t* pd = pdata_fields->phead;
+	int idx = 0;
+	int hlen = pstate->pheader_keeper->pkeys->length;
+	int dlen = pdata_fields->length;
+
+	// Process fields up to minimum of header length and data length
+	// Note that pd->pnext can be non-null due to pointer-reuse semantics of rslls,
+	// so use list-length attributes for end-of-list check.
+	for (idx = 0; idx < hlen && idx < dlen; idx++, ph = ph->pnext, pd = pd->pnext) {
+		// Transfer pointer-free responsibility from the rslls to the lrec object
+		lrec_put_ext(prec, ph->value, pd->value, pd->free_flag, pd->quote_flag);
+		pd->free_flag = 0;
+	}
+
+	if (hlen > dlen) {
+		// Header is longer. Empty-fill the remaining data fields.
+		// E.g. if the input looks like
+		//   a,b,c,d <-- header
+		//   1,2     <-- data
+		// then put c="", d="".
+		for ( ; idx < hlen; idx++, ph = ph->pnext) {
+			lrec_put_ext(prec, ph->value, "", NO_FREE, 0);
+		}
+	} else {
+		// Data is longer. Use positional indices to label the remaining data fields.
+		for ( ; idx < dlen; idx++, pd = pd->pnext) {
+			char key_free_flags = 0;
+			char* key = low_int_to_string(idx+1, &key_free_flags);
+			char value_free_flags = pd->free_flag;
+			// Transfer pointer-free responsibility from the rslls to the lrec object
+			lrec_put_ext(prec, key, pd->value, key_free_flags | value_free_flags, pd->quote_flag);
+			pd->free_flag = 0;
+		}
+	}
+
+	return prec;
+}
+
+// ----------------------------------------------------------------
+static lrec_t* paste_header_and_data_rectangular(lrec_reader_mmap_csv_state_t* pstate, rslls_t* pdata_fields,
+	context_t* pctx)
+{
 	if (pstate->pheader_keeper->pkeys->length != pdata_fields->length) {
 		fprintf(stderr, "%s: Header/data length mismatch (%llu != %llu) at file \"%s\" line %lld.\n",
 			MLR_GLOBALS.bargv0, pstate->pheader_keeper->pkeys->length, pdata_fields->length,
