@@ -35,18 +35,17 @@
 // ----------------------------------------------------------------
 #define STRING_BUILDER_INIT_SIZE 1024
 
-// AKA "token"
-#define EOF_STRIDX           0x2000
-#define IRS_STRIDX           0x2001
-#define IFS_EOF_STRIDX       0x2002
-#define IFS_STRIDX           0x2003
-#define DQUOTE_STRIDX        0x2004
-#define DQUOTE_IRS_STRIDX    0x2005
-#define DQUOTE_IRS2_STRIDX   0x2006 // alternate line-ending for autodetect LF/CRLF
-#define DQUOTE_IFS_STRIDX    0x2007
-#define DQUOTE_EOF_STRIDX    0x2008
-#define DQUOTE_DQUOTE_STRIDX 0x2009
-#define UTF8_BOM_STRIDX      0x200b
+#define EOF_TOKEN           0x2000
+#define IRS_TOKEN           0x2001
+#define IFS_EOF_TOKEN       0x2002
+#define IFS_TOKEN           0x2003
+#define DQUOTE_TOKEN        0x2004
+#define DQUOTE_IRS_TOKEN    0x2005
+#define DQUOTE_IRS2_TOKEN   0x2006 // alternate line-ending for autodetect LF/CRLF
+#define DQUOTE_IFS_TOKEN    0x2007
+#define DQUOTE_EOF_TOKEN    0x2008
+#define DQUOTE_DQUOTE_TOKEN 0x2009
+#define UTF8_BOM_TOKEN      0x200b
 
 #define UTF8_BOM "\xef\xbb\xbf"
 #define UTF8_BOM_LENGTH 3
@@ -86,7 +85,8 @@ typedef struct _lrec_reader_stdio_csv_state_t {
 	parse_trie_t*       pdquote_parse_trie;
 
 	int                 expect_header_line_next;
-	int                 use_implicit_header;
+	int                 use_implicit_csv_header;
+	int                 allow_ragged_csv_input;
 	header_keeper_t*    pheader_keeper;
 	lhmslv_t*           pheader_keepers;
 
@@ -99,14 +99,16 @@ static int     lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* p
 	context_t* pctx, int is_header);
 static lrec_t* paste_indices_and_data(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pdata_fields,
 	context_t* pctx);
-static lrec_t* paste_header_and_data(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pdata_fields,
+static lrec_t* paste_header_and_data_ragged(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pdata_fields,
+	context_t* pctx);
+static lrec_t* paste_header_and_data_rectangular(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pdata_fields,
 	context_t* pctx);
 static void*   lrec_reader_stdio_csv_open(void* pvstate, char* prepipe, char* filename);
 static void    lrec_reader_stdio_csv_close(void* pvstate, void* pvhandle, char* prepipe);
 
 // ----------------------------------------------------------------
-lrec_reader_t* lrec_reader_stdio_csv_alloc(char* irs, char* ifs, int use_implicit_header,
-	comment_handling_t comment_handling, char* comment_string)
+lrec_reader_t* lrec_reader_stdio_csv_alloc(char* irs, char* ifs, int use_implicit_csv_header,
+	int allow_ragged_csv_input, comment_handling_t comment_handling, char* comment_string)
 {
 	lrec_reader_t* plrec_reader = mlr_malloc_or_die(sizeof(lrec_reader_t));
 
@@ -138,33 +140,33 @@ lrec_reader_t* lrec_reader_stdio_csv_alloc(char* irs, char* ifs, int use_implici
 
 	// Parse trie for UTF-8 BOM
 	pstate->putf8_bom_parse_trie = parse_trie_alloc();
-	parse_trie_add_string(pstate->putf8_bom_parse_trie, UTF8_BOM, UTF8_BOM_STRIDX);
+	parse_trie_add_string(pstate->putf8_bom_parse_trie, UTF8_BOM, UTF8_BOM_TOKEN);
 
 	// Parse trie for non-double-quoted fields
 	pstate->pno_dquote_parse_trie = parse_trie_alloc();
-	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->eof,     EOF_STRIDX);
-	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->irs,     IRS_STRIDX);
-	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->ifs_eof, IFS_EOF_STRIDX);
-	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->ifs,     IFS_STRIDX);
-	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->dquote,  DQUOTE_STRIDX);
+	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->eof,     EOF_TOKEN);
+	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->irs,     IRS_TOKEN);
+	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->ifs_eof, IFS_EOF_TOKEN);
+	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->ifs,     IFS_TOKEN);
+	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->dquote,  DQUOTE_TOKEN);
 
 	// Parse trie for double-quoted fields
 	pstate->pdquote_parse_trie = parse_trie_alloc();
 	if (pstate->do_auto_line_term) {
 		pstate->dquote_irs  = mlr_paste_2_strings("\"", "\n");
 		pstate->dquote_irs2 = mlr_paste_2_strings("\"", "\r\n");
-		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs,  DQUOTE_IRS_STRIDX);
-		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs2, DQUOTE_IRS2_STRIDX);
+		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs,  DQUOTE_IRS_TOKEN);
+		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs2, DQUOTE_IRS2_TOKEN);
 	} else {
 		pstate->dquote_irs  = mlr_paste_2_strings("\"", pstate->irs);
 		pstate->dquote_irs2 = NULL;
-		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs, DQUOTE_IRS_STRIDX);
+		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs, DQUOTE_IRS_TOKEN);
 	}
-	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->eof,           EOF_STRIDX);
-	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs,    DQUOTE_IRS_STRIDX);
-	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_ifs,    DQUOTE_IFS_STRIDX);
-	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_eof,    DQUOTE_EOF_STRIDX);
-	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_dquote, DQUOTE_DQUOTE_STRIDX);
+	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->eof,           EOF_TOKEN);
+	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs,    DQUOTE_IRS_TOKEN);
+	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_ifs,    DQUOTE_IFS_TOKEN);
+	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_eof,    DQUOTE_EOF_TOKEN);
+	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_dquote, DQUOTE_DQUOTE_TOKEN);
 
 
 	pstate->pfields = rslls_alloc();
@@ -175,8 +177,9 @@ lrec_reader_t* lrec_reader_stdio_csv_alloc(char* irs, char* ifs, int use_implici
 		pstate->pno_dquote_parse_trie->maxlen,
 		pstate->pdquote_parse_trie->maxlen));
 
-	pstate->expect_header_line_next   = use_implicit_header ? FALSE : TRUE;
-	pstate->use_implicit_header       = use_implicit_header;
+	pstate->expect_header_line_next   = use_implicit_csv_header ? FALSE : TRUE;
+	pstate->use_implicit_csv_header   = use_implicit_csv_header;
+	pstate->allow_ragged_csv_input    = allow_ragged_csv_input;
 	pstate->pheader_keeper            = NULL;
 	pstate->pheader_keepers           = lhmslv_alloc();
 
@@ -217,7 +220,7 @@ static void lrec_reader_stdio_csv_free(lrec_reader_t* preader) {
 static void lrec_reader_stdio_csv_sof(void* pvstate, void* pvhandle) {
 	lrec_reader_stdio_csv_state_t* pstate = pvstate;
 	pstate->ilno = 0LL;
-	pstate->expect_header_line_next = pstate->use_implicit_header ? FALSE : TRUE;
+	pstate->expect_header_line_next = pstate->use_implicit_csv_header ? FALSE : TRUE;
 }
 
 // ----------------------------------------------------------------
@@ -259,7 +262,7 @@ static lrec_t* lrec_reader_stdio_csv_process(void* pvstate, void* pvhandle, cont
 
 			slls_t* pheader_fields = slls_alloc();
 			int i = 0;
-			for (rsllse_t* pe = pstate->pfields->phead; i < pstate->pfields->length && pe != NULL; pe = pe->pnext) {
+			for (rsllse_t* pe = pstate->pfields->phead; i < pstate->pfields->length && pe != NULL; pe = pe->pnext, i++) {
 				if (*pe->value == 0) {
 					fprintf(stderr, "%s: unacceptable empty CSV key at file \"%s\" line %lld.\n",
 						MLR_GLOBALS.bargv0, pctx->filename, pstate->ilno);
@@ -319,9 +322,12 @@ static lrec_t* lrec_reader_stdio_csv_process(void* pvstate, void* pvhandle, cont
 			}
 		}
 
-		lrec_t* prec =  pstate->use_implicit_header
+
+		lrec_t* prec = pstate->use_implicit_csv_header
 			? paste_indices_and_data(pstate, pstate->pfields, pctx)
-			: paste_header_and_data(pstate, pstate->pfields, pctx);
+			: pstate->allow_ragged_csv_input
+			? paste_header_and_data_ragged(pstate, pstate->pfields, pctx)
+			: paste_header_and_data_rectangular(pstate, pstate->pfields, pctx);
 		rslls_reset(pstate->pfields);
 		return prec;
 	}
@@ -330,7 +336,10 @@ static lrec_t* lrec_reader_stdio_csv_process(void* pvstate, void* pvhandle, cont
 static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pfields,
 	context_t* pctx, int is_header)
 {
-	int rc, stridx, matchlen, record_done, field_done;
+	int rc, token = 0, matchlen = 0, record_done = FALSE, field_done = FALSE;
+#ifdef DEBUG_PARSER
+	char c = 0;
+#endif
 	peek_file_reader_t* pfr = pstate->pfr;
 	string_builder_t*   psb = pstate->psb;
 	char* field = NULL;
@@ -348,11 +357,13 @@ static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstat
 		pfr_buffer_by(pfr, UTF8_BOM_LENGTH);
 		int rc = parse_trie_ring_match(pstate->putf8_bom_parse_trie,
 			pfr->peekbuf, pfr->sob, pfr->npeeked, pfr->peekbuflenmask,
-			&stridx, &matchlen);
+			&token, &matchlen);
 #ifdef DEBUG_PARSER
-		printf("RC=%d stridx=0x%04x matchlen=%d\n", rc, stridx, matchlen);
+		if (rc) {
+			printf("BOMRC=%d token=0x%04x matchlen=%d\n", rc, token, matchlen);
+		}
 #endif
-		if (rc == TRUE && stridx == UTF8_BOM_STRIDX) {
+		if (rc == TRUE && token == UTF8_BOM_TOKEN) {
 			pfr_advance_by(pfr, matchlen);
 		}
 	}
@@ -370,33 +381,34 @@ static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstat
 
 				rc = parse_trie_ring_match(pstate->pno_dquote_parse_trie,
 					pfr->peekbuf, pfr->sob, pfr->npeeked, pfr->peekbuflenmask,
-					&stridx, &matchlen);
+					&token, &matchlen);
 #ifdef DEBUG_PARSER
 				pfr_print(pfr);
 #endif
 				if (rc) {
 #ifdef DEBUG_PARSER
-					printf("RC=%d stridx=0x%04x matchlen=%d\n", rc, stridx, matchlen);
+					printf("DQ=0 RC=%d token=0x%04x matchlen=%d\n", rc, token, matchlen);
 #endif
-					switch(stridx) {
-					case EOF_STRIDX: // end of record
+					switch(token) {
+					case EOF_TOKEN: // end of record
 						rslls_append(pfields, sb_finish(psb), FREE_ENTRY_VALUE, 0);
 						field_done  = TRUE;
 						record_done = TRUE;
 						break;
-					case IFS_EOF_STRIDX:
-						fprintf(stderr, "%s: syntax error: record-ending field separator at line %lld.\n",
-							MLR_GLOBALS.bargv0, pstate->ilno);
-						exit(1);
+					case IFS_EOF_TOKEN: // end of record, last field is empty
+						rslls_append(pfields, sb_finish(psb), FREE_ENTRY_VALUE, 0);
+						rslls_append(pfields, "", NO_FREE, 0);
+						field_done  = TRUE;
+						record_done = TRUE;
 						break;
-					case IFS_STRIDX: // end of field
+					case IFS_TOKEN: // end of field
 						rslls_append(pfields, sb_finish(psb), FREE_ENTRY_VALUE, 0);
 						field_done  = TRUE;
 						break;
-					case IRS_STRIDX: // end of record
+					case IRS_TOKEN: // end of record
 						field = sb_finish_with_length(psb, &field_length);
 
-						// The line-ending '\n' won't be included in the field buffer.
+						// The line-ending '\n' wonlt be included in the field buffer.
 						if (pstate->do_auto_line_term) {
 							if (field_length > 0 && field[field_length-1] == '\r') {
 								field[field_length-1] = 0;
@@ -410,21 +422,21 @@ static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstat
 						field_done  = TRUE;
 						record_done = TRUE;
 						break;
-					case DQUOTE_STRIDX: // CSV syntax error: fields containing quotes must be fully wrapped in quotes
+					case DQUOTE_TOKEN: // CSV syntax error: fields containing quotes must be fully wrapped in quotes
 						fprintf(stderr, "%s: syntax error: unwrapped double quote at line %lld.\n",
 							MLR_GLOBALS.bargv0, pstate->ilno);
 						exit(1);
 						break;
 					default:
 						fprintf(stderr, "%s: internal coding error: unexpected token %d at line %lld.\n",
-							MLR_GLOBALS.bargv0, stridx, pstate->ilno);
+							MLR_GLOBALS.bargv0, token, pstate->ilno);
 						exit(1);
 						break;
 					}
 					pfr_advance_by(pfr, matchlen);
 				} else {
 #ifdef DEBUG_PARSER
-					char c = pfr_read_char(pfr);
+					c = pfr_read_char(pfr);
 					printf("CHAR=%c [%02x]\n", isprint((unsigned char)c) ? c : ' ', (unsigned)c);
 					sb_append_char(psb, c);
 #else
@@ -445,26 +457,31 @@ static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstat
 
 				rc = parse_trie_ring_match(pstate->pdquote_parse_trie,
 					pfr->peekbuf, pfr->sob, pfr->npeeked, pfr->peekbuflenmask,
-					&stridx, &matchlen);
-
+					&token, &matchlen);
+#ifdef DEBUG_PARSER
+				pfr_print(pfr);
+#endif
 				if (rc) {
-					switch(stridx) {
-					case EOF_STRIDX: // end of record
+#ifdef DEBUG_PARSER
+					printf("DQ=1 RC=%d token=0x%04x matchlen=%d\n", rc, token, matchlen);
+#endif
+					switch(token) {
+					case EOF_TOKEN: // end of record
 						fprintf(stderr, "%s: unmatched double quote at line %lld.\n",
 							MLR_GLOBALS.bargv0, pstate->ilno);
 						exit(1);
 						break;
-					case DQUOTE_EOF_STRIDX: // end of record
+					case DQUOTE_EOF_TOKEN: // end of record
 						rslls_append(pfields, sb_finish(psb), FREE_ENTRY_VALUE, FIELD_QUOTED_ON_INPUT);
 						field_done  = TRUE;
 						record_done = TRUE;
 						break;
-					case DQUOTE_IFS_STRIDX: // end of field
+					case DQUOTE_IFS_TOKEN: // end of field
 						rslls_append(pfields, sb_finish(psb), FREE_ENTRY_VALUE, FIELD_QUOTED_ON_INPUT);
 						field_done  = TRUE;
 						break;
-					case DQUOTE_IRS_STRIDX: // end of record
-					case DQUOTE_IRS2_STRIDX: // end of record
+					case DQUOTE_IRS_TOKEN: // end of record
+					case DQUOTE_IRS2_TOKEN: // end of record
 
 						field = sb_finish_with_length(psb, &field_length);
 
@@ -482,18 +499,30 @@ static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstat
 						field_done  = TRUE;
 						record_done = TRUE;
 						break;
-					case DQUOTE_DQUOTE_STRIDX: // RFC-4180 CSV: "" inside a dquoted field is an escape for "
+					case DQUOTE_DQUOTE_TOKEN: // RFC-4180 CSV: "" inside a dquoted field is an escape for "
+#ifdef DEBUG_PARSER
+						c = pstate->dquote[0];
+						printf("CHAR=%c [%02x]\n", isprint((unsigned char)c) ? c : ' ', (unsigned)c);
+						sb_append_char(psb, c);
+#else
 						sb_append_char(psb, pstate->dquote[0]);
+#endif
 						break;
 					default:
 						fprintf(stderr, "%s: internal coding error: unexpected token %d at line %lld.\n",
-							MLR_GLOBALS.bargv0, stridx, pstate->ilno);
+							MLR_GLOBALS.bargv0, token, pstate->ilno);
 						exit(1);
 						break;
 					}
 					pfr_advance_by(pfr, matchlen);
 				} else {
+#ifdef DEBUG_PARSER
+					c = pfr_read_char(pfr);
+					printf("CHAR=%c [%02x]\n", isprint((unsigned char)c) ? c : ' ', (unsigned)c);
+					sb_append_char(psb, c);
+#else
 					sb_append_char(psb, pfr_read_char(pfr));
+#endif
 				}
 			}
 
@@ -522,7 +551,51 @@ static lrec_t* paste_indices_and_data(lrec_reader_stdio_csv_state_t* pstate, rsl
 }
 
 // ----------------------------------------------------------------
-static lrec_t* paste_header_and_data(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pdata_fields,
+static lrec_t* paste_header_and_data_ragged(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pdata_fields,
+	context_t* pctx)
+{
+	lrec_t* prec = lrec_unbacked_alloc();
+	sllse_t* ph  = pstate->pheader_keeper->pkeys->phead;
+	rsllse_t* pd = pdata_fields->phead;
+	int idx = 0;
+	int hlen = pstate->pheader_keeper->pkeys->length;
+	int dlen = pdata_fields->length;
+
+	// Process fields up to minimum of header length and data length
+	// Note that pd->pnext can be non-null due to pointer-reuse semantics of rslls,
+	// so use list-length attributes for end-of-list check.
+	for (idx = 0; idx < hlen && idx < dlen; idx++, ph = ph->pnext, pd = pd->pnext) {
+		// Transfer pointer-free responsibility from the rslls to the lrec object
+		lrec_put_ext(prec, ph->value, pd->value, pd->free_flag, pd->quote_flag);
+		pd->free_flag = 0;
+	}
+
+	if (hlen > dlen) {
+		// Header is longer. Empty-fill the remaining data fields.
+		// E.g. if the input looks like
+		//   a,b,c,d <-- header
+		//   1,2     <-- data
+		// then put c="", d="".
+		for ( ; idx < hlen; idx++, ph = ph->pnext) {
+			lrec_put_ext(prec, ph->value, "", NO_FREE, 0);
+		}
+	} else {
+		// Data is longer. Use positional indices to label the remaining data fields.
+		for ( ; idx < dlen; idx++, pd = pd->pnext) {
+			char key_free_flags = 0;
+			char* key = low_int_to_string(idx+1, &key_free_flags);
+			char value_free_flags = pd->free_flag;
+			// Transfer pointer-free responsibility from the rslls to the lrec object
+			lrec_put_ext(prec, key, pd->value, key_free_flags | value_free_flags, pd->quote_flag);
+			pd->free_flag = 0;
+		}
+	}
+
+	return prec;
+}
+
+// ----------------------------------------------------------------
+static lrec_t* paste_header_and_data_rectangular(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pdata_fields,
 	context_t* pctx)
 {
 	if (pstate->pheader_keeper->pkeys->length != pdata_fields->length) {
