@@ -16,6 +16,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include "cli/comment_handling.h"
 #include "lib/mlr_globals.h"
 #include "lib/mlrutil.h"
 #include "input/file_ingestor_stdio.h"
@@ -36,8 +37,11 @@ typedef struct _lrec_reader_stdio_json_state_t {
 	sllv_t* precords;
 	char* input_json_flatten_separator;
 	json_array_ingest_t json_array_ingest;
+	char* specified_line_term;
 	int do_auto_line_term;
 	char* detected_line_term;
+	comment_handling_t comment_handling;
+	char* comment_string;
 } lrec_reader_stdio_json_state_t;
 
 static void    lrec_reader_stdio_json_free(lrec_reader_t* preader);
@@ -45,8 +49,8 @@ static void    lrec_reader_stdio_json_sof(void* pvstate, void* pvhandle);
 static lrec_t* lrec_reader_stdio_json_process(void* pvstate, void* pvhandle, context_t* pctx);
 
 // ----------------------------------------------------------------
-lrec_reader_t* lrec_reader_stdio_json_alloc(char* input_json_flatten_separator, json_array_ingest_t json_array_ingest,
-	char* line_term)
+lrec_reader_t* lrec_reader_stdio_json_alloc(char* input_json_flatten_separator, json_array_ingest_t json_array_ingest, char* line_term,
+	comment_handling_t comment_handling, char* comment_string)
 {
 	lrec_reader_t* plrec_reader = mlr_malloc_or_die(sizeof(lrec_reader_t));
 
@@ -55,8 +59,11 @@ lrec_reader_t* lrec_reader_stdio_json_alloc(char* input_json_flatten_separator,
 	pstate->precords                     = sllv_alloc();
 	pstate->input_json_flatten_separator = input_json_flatten_separator;
 	pstate->json_array_ingest            = json_array_ingest;
+	pstate->specified_line_term          = line_term;
 	pstate->do_auto_line_term            = FALSE;
 	pstate->detected_line_term           = "\n"; // xxx adapt to MLR_GLOBALS/ctx-const for Windows port
+	pstate->comment_handling             = comment_handling;
+	pstate->comment_string               = comment_string;
 
 	if (streq(line_term, "auto")) {
 		pstate->do_auto_line_term = TRUE;
@@ -143,37 +150,51 @@ static void lrec_reader_stdio_json_sof(void* pvstate, void* pvhandle) {
 		}
 	}
 
-	while (TRUE) {
+	// Skip comments. For JSON, we ingest the entire blob, this is a matter of finding and iterating over lines.
+	// Miller data comments must be at start of line.
+	if (pstate->comment_handling != COMMENTS_ARE_DATA) {
+		char* line_term = pstate->specified_line_term;
+		if (pstate->do_auto_line_term && detected_line_term != NULL)
+			line_term = detected_line_term;
+		mlr_json_strip_comments(phandle->sof, phandle->eof,
+			pstate->comment_handling, pstate->comment_string, line_term);
+	}
+	mlr_json_end_strip(phandle->sof, &phandle->eof);
+	length = phandle->eof - phandle->sof;
 
-		parsed_top_level_json = json_parse(item_start, length, error_buf, &item_start);
+	if (length > 0) {
+		while (TRUE) {
 
-		if (parsed_top_level_json == NULL) {
-			fprintf(stderr, "%s: Unable to parse JSON data: %s\n", MLR_GLOBALS.bargv0, error_buf);
-			exit(1);
-		}
+			parsed_top_level_json = json_parse(item_start, length, error_buf, &item_start);
 
-		// The lrecs have their string pointers pointing into the parsed-JSON objects (for
-		// efficiency) so it's important we not free the latter until our free method.
-		if (!reference_json_objects_as_lrecs(pstate->precords, parsed_top_level_json,
-			pstate->input_json_flatten_separator, pstate->json_array_ingest))
-		{
-			fprintf(stderr, "%s: Unable to parse JSON data.\n", MLR_GLOBALS.bargv0);
-			exit(1);
-		}
+			if (parsed_top_level_json == NULL) {
+				fprintf(stderr, "%s: Unable to parse JSON data: %s\n", MLR_GLOBALS.bargv0, error_buf);
+				exit(1);
+			}
 
-		if (item_start == NULL)
-			break;
-		if (*item_start == 0)
-			break;
-		length -= (item_start - json_input);
-		json_input = item_start;
-		// json_parse goes up to the '\r' or '\n' (whichever is found first) on the first
-		// parse, then keeps going from there on the next. E.g. in the CRLF case it
-		// consumes the CR at the end of the first read and consumes the LF at the start
-		// of the second, and so on. After the very last parse, we need to here consume
-		// the final '\n' which is (by itself) a parse error.
-		if (length == 1 && *(char*)json_input == '\n') {
-			break;
+			// The lrecs have their string pointers pointing into the parsed-JSON objects (for
+			// efficiency) so it's important we not free the latter until our free method.
+			if (!reference_json_objects_as_lrecs(pstate->precords, parsed_top_level_json,
+				pstate->input_json_flatten_separator, pstate->json_array_ingest))
+			{
+				fprintf(stderr, "%s: Unable to parse JSON data.\n", MLR_GLOBALS.bargv0);
+				exit(1);
+			}
+
+			if (item_start == NULL)
+				break;
+			if (*item_start == 0)
+				break;
+			length -= (item_start - json_input);
+			json_input = item_start;
+			// json_parse goes up to the '\r' or '\n' (whichever is found first) on the first
+			// parse, then keeps going from there on the next. E.g. in the CRLF case it
+			// consumes the CR at the end of the first read and consumes the LF at the start
+			// of the second, and so on. After the very last parse, we need to here consume
+			// the final '\n' which is (by itself) a parse error.
+			if (length == 1 && *(char*)json_input == '\n') {
+				break;
+			}
 		}
 	}
 	if (detected_line_term != NULL) {
