@@ -8,6 +8,7 @@
 // ================================================================
 
 #include <stdlib.h>
+#include "cli/comment_handling.h"
 #include "lib/mlrutil.h"
 #include "input/file_reader_stdio.h"
 #include "input/line_readers.h"
@@ -19,6 +20,8 @@ typedef struct _lrec_reader_stdio_nidx_state_t {
 	int    irslen;
 	int    ifslen;
 	int    allow_repeat_ifs;
+	comment_handling_t comment_handling;
+	char*  comment_string;
 	size_t line_length;
 } lrec_reader_stdio_nidx_state_t;
 
@@ -34,7 +37,9 @@ static lrec_t* lrec_reader_stdio_nidx_process_multi_irs_single_ifs(void* pvstate
 static lrec_t* lrec_reader_stdio_nidx_process_multi_irs_multi_ifs(void* pvstate, void* pvhandle, context_t* pctx);
 
 // ----------------------------------------------------------------
-lrec_reader_t* lrec_reader_stdio_nidx_alloc(char* irs, char* ifs, int allow_repeat_ifs) {
+lrec_reader_t* lrec_reader_stdio_nidx_alloc(char* irs, char* ifs, int allow_repeat_ifs,
+	comment_handling_t comment_handling, char* comment_string)
+{
 	lrec_reader_t* plrec_reader = mlr_malloc_or_die(sizeof(lrec_reader_t));
 
 	lrec_reader_stdio_nidx_state_t* pstate = mlr_malloc_or_die(sizeof(lrec_reader_stdio_nidx_state_t));
@@ -43,6 +48,8 @@ lrec_reader_t* lrec_reader_stdio_nidx_alloc(char* irs, char* ifs, int allow_repe
 	pstate->irslen           = strlen(irs);
 	pstate->ifslen           = strlen(ifs);
 	pstate->allow_repeat_ifs = allow_repeat_ifs;
+	pstate->comment_handling = comment_handling;
+	pstate->comment_string   = comment_string;
 	// This is used to track nominal line length over the file read. Bootstrap with a default length.
 	pstate->line_length      = MLR_ALLOC_READ_LINE_INITIAL_SIZE;
 
@@ -88,8 +95,11 @@ static lrec_t* lrec_reader_stdio_nidx_process_single_irs_single_ifs_auto_line_te
 	FILE* input_stream = pvhandle;
 	lrec_reader_stdio_nidx_state_t* pstate = pvstate;
 
-	char* line = mlr_alloc_read_line_single_delimiter(input_stream, pstate->irs[0],
-		&pstate->line_length, TRUE, pctx);
+	char* line = pstate->comment_handling == COMMENTS_ARE_DATA
+		? mlr_alloc_read_line_single_delimiter(input_stream, pstate->irs[0],
+			&pstate->line_length, TRUE, pctx)
+		: mlr_alloc_read_line_single_delimiter_stripping_comments(input_stream, pstate->irs[0],
+			&pstate->line_length, TRUE, pstate->comment_handling, pstate->comment_string, pctx);
 
 	if (line == NULL) {
 		return NULL;
@@ -102,8 +112,11 @@ static lrec_t* lrec_reader_stdio_nidx_process_single_irs_multi_ifs_auto_line_ter
 	FILE* input_stream = pvhandle;
 	lrec_reader_stdio_nidx_state_t* pstate = pvstate;
 
-	char* line = mlr_alloc_read_line_single_delimiter(input_stream, pstate->irs[0],
-		&pstate->line_length, TRUE, pctx);
+	char* line = pstate->comment_handling == COMMENTS_ARE_DATA
+		? mlr_alloc_read_line_single_delimiter(input_stream, pstate->irs[0],
+			&pstate->line_length, TRUE, pctx)
+		: mlr_alloc_read_line_single_delimiter_stripping_comments(input_stream, pstate->irs[0],
+			&pstate->line_length, TRUE, pstate->comment_handling, pstate->comment_string, pctx);
 
 	if (line == NULL) {
 		return NULL;
@@ -116,8 +129,11 @@ static lrec_t* lrec_reader_stdio_nidx_process_single_irs_single_ifs(void* pvstat
 	FILE* input_stream = pvhandle;
 	lrec_reader_stdio_nidx_state_t* pstate = pvstate;
 
-	char* line = mlr_alloc_read_line_single_delimiter(input_stream, pstate->irs[0],
-		&pstate->line_length, FALSE, pctx);
+	char* line = pstate->comment_handling == COMMENTS_ARE_DATA
+		? mlr_alloc_read_line_single_delimiter(input_stream, pstate->irs[0],
+			&pstate->line_length, FALSE, pctx)
+		: mlr_alloc_read_line_single_delimiter_stripping_comments(input_stream, pstate->irs[0],
+			&pstate->line_length, FALSE, pstate->comment_handling, pstate->comment_string, pctx);
 
 	if (line == NULL)
 		return NULL;
@@ -129,8 +145,11 @@ static lrec_t* lrec_reader_stdio_nidx_process_single_irs_multi_ifs(void* pvstate
 	FILE* input_stream = pvhandle;
 	lrec_reader_stdio_nidx_state_t* pstate = pvstate;
 
-	char* line = mlr_alloc_read_line_single_delimiter(input_stream, pstate->irs[0],
-		&pstate->line_length, FALSE, pctx);
+	char* line = pstate->comment_handling == COMMENTS_ARE_DATA
+		? mlr_alloc_read_line_single_delimiter(input_stream, pstate->irs[0],
+			&pstate->line_length, FALSE, pctx)
+		: mlr_alloc_read_line_single_delimiter_stripping_comments(input_stream, pstate->irs[0],
+			&pstate->line_length, FALSE, pstate->comment_handling, pstate->comment_string, pctx);
 
 	if (line == NULL)
 		return NULL;
@@ -141,8 +160,11 @@ static lrec_t* lrec_reader_stdio_nidx_process_single_irs_multi_ifs(void* pvstate
 static lrec_t* lrec_reader_stdio_nidx_process_multi_irs_single_ifs(void* pvstate, void* pvhandle, context_t* pctx) {
 	lrec_reader_stdio_nidx_state_t* pstate = pvstate;
 	FILE* input_stream = pvhandle;
-	char* line = mlr_alloc_read_line_multiple_delimiter(input_stream, pstate->irs, pstate->irslen,
-		&pstate->line_length);
+	char* line = pstate->comment_handling == COMMENTS_ARE_DATA
+		? mlr_alloc_read_line_multiple_delimiter(input_stream, pstate->irs, pstate->irslen,
+			&pstate->line_length)
+		: mlr_alloc_read_line_multiple_delimiter_stripping_comments(input_stream, pstate->irs, pstate->irslen,
+			&pstate->line_length, pstate->comment_handling, pstate->comment_string);
 	if (line == NULL)
 		return NULL;
 	else
@@ -152,8 +174,11 @@ static lrec_t* lrec_reader_stdio_nidx_process_multi_irs_single_ifs(void* pvstate
 static lrec_t* lrec_reader_stdio_nidx_process_multi_irs_multi_ifs(void* pvstate, void* pvhandle, context_t* pctx) {
 	lrec_reader_stdio_nidx_state_t* pstate = pvstate;
 	FILE* input_stream = pvhandle;
-	char* line = mlr_alloc_read_line_multiple_delimiter(input_stream, pstate->irs, pstate->irslen,
-		&pstate->line_length);
+	char* line = pstate->comment_handling == COMMENTS_ARE_DATA
+		? mlr_alloc_read_line_multiple_delimiter(input_stream, pstate->irs, pstate->irslen,
+			&pstate->line_length)
+		: mlr_alloc_read_line_multiple_delimiter_stripping_comments(input_stream, pstate->irs, pstate->irslen,
+			&pstate->line_length, pstate->comment_handling, pstate->comment_string);
 	if (line == NULL)
 		return NULL;
 	else
