@@ -173,6 +173,13 @@ char* mlr_alloc_hexfmt_from_ll(long long value) {
 	return string;
 }
 
+char* mlr_alloc_string_from_string_and_format(char* old_value, char* fmt) {
+	int n = snprintf(NULL, 0, fmt, old_value);
+	char* new_value = mlr_malloc_or_die(n+1);
+	sprintf(new_value, fmt, old_value);
+	return new_value;
+}
+
 double mlr_double_from_string_or_die(char* string) {
 	double d;
 	if (!mlr_try_float_from_string(string, &d)) {
@@ -602,3 +609,30 @@ char* alloc_suffixed_temp_file_name(char* filename) {
 
 	return output;
 }
+
+// ----------------------------------------------------------------
+// The convention for argv-style arrays is that they're null-terminated.
+// So we loop through once to find the length.
+char** copy_argv(char** argv) {
+	int length = 0;
+	int argi;
+	for (argi = 0; argv[argi]; argi++) {
+		length++;
+	}
+
+	char** copy = mlr_malloc_or_die((length + 1) * sizeof(char*));
+	for (argi = 0; argv[argi]; argi++) {
+		copy[argi] = mlr_strdup_or_die(argv[argi]);
+	}
+
+	copy[length] = 0;
+
+	return copy;
+}
+
+void free_argv_copy(char** copy) {
+	for (int argi = 0; copy[argi]; argi++) {
+		free(copy[argi]);
+	}
+	free(copy);
+}
