@@ -178,6 +178,13 @@ static inline mv_t i_ii_urandint_func(mv_t* pval1, mv_t* pval2) {
 	return mv_from_int(u);
 }
 
+static inline mv_t f_ff_urandrange_func(mv_t* pval1, mv_t* pval2) {
+	double lo = pval1->u.fltv;
+	double hi = pval2->u.fltv;
+	double u  = lo + (hi - lo) * get_mtrand_double();
+	return mv_from_float(u);
+}
+
 static inline mv_t i_ii_bitwise_lsh_func(mv_t* pval1, mv_t* pval2) {
 	return mv_from_int(pval1->u.intv << pval2->u.intv);
 }
@@ -199,12 +206,15 @@ mv_t i_s_strlen_func(mv_t* pval1);
 mv_t s_x_typeof_func(mv_t* pval1);
 mv_t s_s_tolower_func(mv_t* pval1);
 mv_t s_s_toupper_func(mv_t* pval1);
+mv_t s_s_capitalize_func(mv_t* pval1);
 mv_t s_s_lstrip_func(mv_t* pval1);
 mv_t s_s_rstrip_func(mv_t* pval1);
 mv_t s_s_strip_func(mv_t* pval1);
 mv_t s_s_collapse_whitespace_func(mv_t* pval1);
 mv_t s_s_clean_whitespace_func(mv_t* pval1);
 
+mv_t s_s_system_func(mv_t* pval1);
+
 mv_t s_xx_dot_func(mv_t* pval1, mv_t* pval2);
 
 mv_t sub_no_precomp_func(mv_t* pval1, mv_t* pval2, mv_t* pval3);
