@@ -17,7 +17,7 @@ static void      mapper_group_like_usage(FILE* o, char* argv0, char* verb);
 static mapper_t* mapper_group_like_parse_cli(int* pargi, int argc, char** argv,
 	cli_reader_opts_t* _, cli_writer_opts_t* __);
 static mapper_t* mapper_group_like_alloc();
-static void      mapper_group_like_free(mapper_t* pmapper);
+static void      mapper_group_like_free(mapper_t* pmapper, context_t* _);
 static sllv_t*   mapper_group_like_process(lrec_t* pinrec, context_t* pctx, void* pvstate);
 
 // ----------------------------------------------------------------
@@ -25,6 +25,7 @@ mapper_setup_t mapper_group_like_setup = {
 	.verb = "group-like",
 	.pusage_func = mapper_group_like_usage,
 	.pparse_func = mapper_group_like_parse_cli,
+	.ignores_input = FALSE,
 };
 
 // ----------------------------------------------------------------
@@ -59,7 +60,7 @@ static mapper_t* mapper_group_like_alloc() {
 	return pmapper;
 }
 
-static void mapper_group_like_free(mapper_t* pmapper) {
+static void mapper_group_like_free(mapper_t* pmapper, context_t* _) {
 	mapper_group_like_state_t* pstate = pmapper->pvstate;
 	// lhmslv_free will free the hashmap keys; we need to free the void-star hashmap values.
 	for (lhmslve_t* pa = pstate->precords_by_key_field_names->phead; pa != NULL; pa = pa->pnext) {
