@@ -107,7 +107,7 @@ static void mapper_join_usage(FILE* o, char* argv0, char* verb) {
 	fprintf(o, "  --ips {pair-separator character}\n");
 	fprintf(o, "  --repifs\n");
 	fprintf(o, "  --repips\n");
-	fprintf(o, "  --use-mmap\n");
+	fprintf(o, "  --mmap\n");
 	fprintf(o, "  --no-mmap\n");
 	fprintf(o, "Please use \"%s --usage-separator-options\" for information on specifying separators.\n",
 		argv0);
