@@ -7,14 +7,15 @@ static void      mapper_nothing_usage(FILE* o, char* argv0, char* verb);
 static mapper_t* mapper_nothing_parse_cli(int* pargi, int argc, char** argv,
 	cli_reader_opts_t* _, cli_writer_opts_t* __);
 static mapper_t* mapper_nothing_alloc();
-static void      mapper_nothing_free(mapper_t* pmapper);
+static void      mapper_nothing_free(mapper_t* pmapper, context_t* _);
 static sllv_t*   mapper_nothing_process(lrec_t* pinrec, context_t* pctx, void* pvstate);
 
 // ----------------------------------------------------------------
 mapper_setup_t mapper_nothing_setup = {
 	.verb = "nothing",
 	.pusage_func = mapper_nothing_usage,
-	.pparse_func = mapper_nothing_parse_cli
+	.pparse_func = mapper_nothing_parse_cli,
+	.ignores_input = FALSE,
 };
 
 // ----------------------------------------------------------------
@@ -44,7 +45,7 @@ static mapper_t* mapper_nothing_alloc(ap_state_t* pargp, int do_counters, char*
 	pmapper->pfree_func    = mapper_nothing_free;
 	return pmapper;
 }
-static void mapper_nothing_free(mapper_t* pmapper) {
+static void mapper_nothing_free(mapper_t* pmapper, context_t* _) {
 	free(pmapper);
 }
 
