@@ -10,7 +10,7 @@
 #include "containers/lhmslv.h"
 #include "containers/lhmsv.h"
 #include "containers/mixutil.h"
-#include "containers/mvfuncs.h"
+#include "lib/mvfuncs.h"
 #include "mapping/mappers.h"
 #include "cli/argparse.h"
 
