@@ -22,14 +22,15 @@ static void      mapper_tail_usage(FILE* o, char* argv0, char* verb);
 static mapper_t* mapper_tail_parse_cli(int* pargi, int argc, char** argv,
 	cli_reader_opts_t* _, cli_writer_opts_t* __);
 static mapper_t* mapper_tail_alloc(ap_state_t* pargp, slls_t* pgroup_by_field_names, unsigned long long tail_count);
-static void      mapper_tail_free(mapper_t* pmapper);
+static void      mapper_tail_free(mapper_t* pmapper, context_t* _);
 static sllv_t*   mapper_tail_process(lrec_t* pinrec, context_t* pctx, void* pvstate);
 
 // ----------------------------------------------------------------
 mapper_setup_t mapper_tail_setup = {
 	.verb = "tail",
 	.pusage_func = mapper_tail_usage,
-	.pparse_func = mapper_tail_parse_cli
+	.pparse_func = mapper_tail_parse_cli,
+	.ignores_input = FALSE,
 };
 
 // ----------------------------------------------------------------
@@ -78,7 +79,7 @@ static mapper_t* mapper_tail_alloc(ap_state_t* pargp, slls_t* pgroup_by_field_na
 	return pmapper;
 }
 
-static void mapper_tail_free(mapper_t* pmapper) {
+static void mapper_tail_free(mapper_t* pmapper, context_t* _) {
 	mapper_tail_state_t* pstate = pmapper->pvstate;
 	if (pstate->pgroup_by_field_names != NULL)
 		slls_free(pstate->pgroup_by_field_names);
@@ -105,7 +106,8 @@ static sllv_t* mapper_tail_process(lrec_t* pinrec, context_t* pctx, void* pvstat
 			sllv_t* precord_list_for_group = lhmslv_get(pstate->precord_lists_by_group, pgroup_by_field_values);
 			if (precord_list_for_group == NULL) {
 				precord_list_for_group = sllv_alloc();
-				lhmslv_put(pstate->precord_lists_by_group, slls_copy(pgroup_by_field_values), precord_list_for_group, FREE_ENTRY_KEY);
+				lhmslv_put(pstate->precord_lists_by_group, slls_copy(pgroup_by_field_values),
+					precord_list_for_group, FREE_ENTRY_KEY);
 			}
 			slls_free(pgroup_by_field_values);
 			if (precord_list_for_group->length >= pstate->tail_count) {
