@@ -16,6 +16,7 @@ extern mapper_setup_t mapper_cut_setup;
 extern mapper_setup_t mapper_decimate_setup;
 extern mapper_setup_t mapper_fill_down_setup;
 extern mapper_setup_t mapper_filter_setup;
+extern mapper_setup_t mapper_format_values_setup;
 extern mapper_setup_t mapper_fraction_setup;
 extern mapper_setup_t mapper_grep_setup;
 extern mapper_setup_t mapper_group_by_setup;
@@ -32,6 +33,7 @@ extern mapper_setup_t mapper_nest_setup;
 extern mapper_setup_t mapper_nothing_setup;
 extern mapper_setup_t mapper_put_setup;
 extern mapper_setup_t mapper_regularize_setup;
+extern mapper_setup_t mapper_remove_empty_columns_setup;
 extern mapper_setup_t mapper_rename_setup;
 extern mapper_setup_t mapper_reorder_setup;
 extern mapper_setup_t mapper_repeat_setup;
@@ -41,7 +43,10 @@ extern mapper_setup_t mapper_sec2gmt_setup;
 extern mapper_setup_t mapper_sec2gmtdate_setup;
 extern mapper_setup_t mapper_seqgen_setup;
 extern mapper_setup_t mapper_shuffle_setup;
+extern mapper_setup_t mapper_skip_trivial_records_setup;
 extern mapper_setup_t mapper_sort_setup;
+// xxx temp for 5.4.0 -- will continue work after
+// extern mapper_setup_t mapper_sort_within_records_setup;
 extern mapper_setup_t mapper_stats1_setup;
 extern mapper_setup_t mapper_stats2_setup;
 extern mapper_setup_t mapper_step_setup;
