@@ -7,8 +7,10 @@ typedef struct _lrec_writer_nidx_state_t {
 	char* ofs;
 } lrec_writer_nidx_state_t;
 
-static void lrec_writer_nidx_free(lrec_writer_t* pwriter);
-static void lrec_writer_nidx_process(void* pvstate, FILE* output_stream, lrec_t* prec);
+static void lrec_writer_nidx_free(lrec_writer_t* pwriter, context_t* pctx);
+static void lrec_writer_nidx_process(void* pvstate, FILE* output_stream, lrec_t* prec, char* ors);
+static void lrec_writer_nidx_process_auto_ors(void* pvstate, FILE* output_stream, lrec_t* prec, context_t* pctx);
+static void lrec_writer_nidx_process_nonauto_ors(void* pvstate, FILE* output_stream, lrec_t* prec, context_t* pctx);
 
 // ----------------------------------------------------------------
 lrec_writer_t* lrec_writer_nidx_alloc(char* ors, char* ofs) {
@@ -19,23 +21,33 @@ lrec_writer_t* lrec_writer_nidx_alloc(char* ors, char* ofs) {
 	pstate->ofs = ofs;
 
 	plrec_writer->pvstate       = (void*)pstate;
-	plrec_writer->pprocess_func = lrec_writer_nidx_process;
+	plrec_writer->pprocess_func = streq(ors, "auto")
+		? lrec_writer_nidx_process_auto_ors
+		: lrec_writer_nidx_process_nonauto_ors;
 	plrec_writer->pfree_func    = lrec_writer_nidx_free;
 
 	return plrec_writer;
 }
 
-static void lrec_writer_nidx_free(lrec_writer_t* pwriter) {
+static void lrec_writer_nidx_free(lrec_writer_t* pwriter, context_t* pctx) {
 	free(pwriter->pvstate);
 	free(pwriter);
 }
 
 // ----------------------------------------------------------------
-static void lrec_writer_nidx_process(void* pvstate, FILE* output_stream, lrec_t* prec) {
+static void lrec_writer_nidx_process_auto_ors(void* pvstate, FILE* output_stream, lrec_t* prec, context_t* pctx) {
+	lrec_writer_nidx_process(pvstate, output_stream, prec, pctx->auto_line_term);
+}
+
+static void lrec_writer_nidx_process_nonauto_ors(void* pvstate, FILE* output_stream, lrec_t* prec, context_t* pctx) {
+	lrec_writer_nidx_state_t* pstate = pvstate;
+	lrec_writer_nidx_process(pvstate, output_stream, prec, pstate->ors);
+}
+
+static void lrec_writer_nidx_process(void* pvstate, FILE* output_stream, lrec_t* prec, char* ors) {
 	if (prec == NULL)
 		return;
 	lrec_writer_nidx_state_t* pstate = pvstate;
-	char* ors = pstate->ors;
 	char* ofs = pstate->ofs;
 
 	int nf = 0;
