@@ -1,10 +1,10 @@
 mlr --from data/rect.txt put -q '
-  ispresent($outer) {
+  is_present($outer) {
     unset @r
   }
   for (k, v in $*) {
     @r[k] = v
   }
-  ispresent($inner1) {
+  is_present($inner1) {
     emit @r
   }'
